# -*- coding: utf-8 -*-
"""
Serial_test.py

Test demo for Bayangtoys-compatible drone.

Sends commands to Arduino Nano via Serial bus which then
instructs the nRF24L01+ 2.4G Wireless Chip to control
the drone.

Commands:
	[arrows] = Pitch
	[W-S] = Throttle up/down
	[A-D] = Yaw

(c) 2017 milankragujevic, based on work from perrytsao and goebish
"""
import serial, time, msvcrt

throttle=1000 # Throttle power 
aileron=1500 # Left-Right pitch
elevator=1500 # Up-Down pitch
rudder=1500 # yaw

tg=25 # Throttle gain
ag=50 # Aileron gain
eg=50 # Elevator gain
rg=50 # Rudder gain

comPort = 'COM4'

try:
	arduino=serial.Serial(comPort, 115200, timeout=.01)
	time.sleep(1)
	while True:
		
		data = arduino.readline()
		if data:
			# Responses from Arduino are [DRONE]
			print "[DRONE]: "+data 
			
		if msvcrt.kbhit():
			key = ord(msvcrt.getch())
			if key == 27: #ESC
				print "[CONTROLLER] ESC exiting"
				break
			if key == 114: #ESC
				print "[CONTROLLER]: R reset"
				throttle=1000 # Throttle power 
				aileron=1500 # Left-Right pitch
				elevator=1500 # Up-Down pitch
				rudder=1500 # yaw
			elif key == 13: # Enter
				print "[CONTROLLER]: Enter"
			elif key == 119: # Less throttle
				throttle+=tg
			elif key == 97: #a
				rudder-=rg		 
			elif key == 115: #s
				throttle-=tg
			elif key == 100: #d
				rudder+=rg
			elif key == 224: # Special keys
				key = ord(msvcrt.getch())
				if key == 80: # Down pitch
					elevator -= eg
				elif key == 72: # Up pitch
					elevator += eg
				elif key == 77: # Right pitch
					aileron += ag
				elif key == 75: # Left pitch
					aileron -= ag			   
			if throttle < 1000:
				throttle = 1000
			if throttle > 1900:
				throttle = 1900
			if aileron < 1000:
				aileron = 1000
			if aileron > 2000:
				aileron = 2000
			if elevator < 1000:
				elevator = 1000
			if elevator > 2000:
				elevator = 2000
			if rudder < 1000:
				rudder = 1000
			if rudder > 2000:
				rudder = 2000
			command = "%i,%i,%i,%i" % (throttle, aileron, elevator, rudder)	 
			# Commands to Arudino are [CONTROLLER]
			print "[CONTROLLER]: " + command 
			arduino.write(command + "\n")

finally:
	# When script exists, reboot Arduino
	# Basically, close the serial connection the re-open it to reset
	arduino.close()
	arduino=serial.Serial(comPort, 115200, timeout=.01)
	# Then close it again so it's not used anymore.
	arduino.close()